/*
 * Scilab ( http://www.scilab.org/ ) - This file is part of Scilab
 * Copyright (C) 2013 - Scilab Enterprises - Antoine ELIAS
 *
 * This file must be used under the terms of the CeCILL.
 * This source file is licensed as described in the file COPYING, which
 * you should have received as part of this distribution.  The terms
 * are also available at
 * http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt
 *
 */

#include "MainObject.hxx"
#include "getInputArgument.hxx"
/*--------------------------------------------------------------------------*/
extern "C" 
{
    #include <stdlib.h>
    #include <api_scilab.h>
	#include <localization.h>
	#include <Scierror.h>
	#include <MALLOC.h>
	#include "gw_xls_link.h"
}

int sci_XLS_callMethod(char *fname,unsigned long /*fname_len*/)
{
    SciErr sciErr;

    int iPropCount = Max(nbInputArgument(pvApiCtx) - 1, 2); //at leat 2 properties
    int* piProperty = NULL;
    wchar_t** pwstProperty = NULL;

    int* piValue = NULL;

    CheckInputArgumentAtLeast(pvApiCtx, 2);
    CheckOutputArgument(pvApiCtx, 1, 1);

    
    //properties list
    pwstProperty = (wchar_t**)MALLOC(sizeof(wchar_t*) * iPropCount);

    for(int i = 0 ; i < iPropCount ; i++)
    {
        int iRhs = i + 1;
        sciErr = getVarAddressFromPosition(pvApiCtx, iRhs, &piProperty);
        if(sciErr.iErr)
        {
            printError(&sciErr, 0);
            Scierror(999, _("%s: unable to get argument #%d\n"), fname, iRhs);
            return 1;
        }

        if(getAllocatedSingleWideString(pvApiCtx, piProperty, &pwstProperty[i]))
        {
            Scierror(999, _("%s: unable to get argument #%d\n"), fname, iRhs);
            return 1;
        }
    }

    VARIANT vData;
    vData.vt = VT_EMPTY;
    if(nbInputArgument(pvApiCtx) > 2)
    {
        //value addr
        getVarAddressFromPosition(pvApiCtx, nbInputArgument(pvApiCtx), &piValue);
        vData = getVariant(piValue);
    }

    ExcelLink *currentObjExcelLink = getObjExcelLink();

    VARIANT vResult;
    VariantClear(&vResult);
    vResult.vt = VT_EMPTY;

    if(nbInputArgument(pvApiCtx) > 3)
    {
        vResult = currentObjExcelLink->getProperty(pwstProperty[0], pwstProperty[1]);
        for(int i = 2 ; i < nbInputArgument(pvApiCtx) - 2 ; i++)
        {
            vResult = currentObjExcelLink->getProperty(vResult, pwstProperty[i]);
        }
    }

    int iArg = (nbInputArgument(pvApiCtx) == 2 ? 0 : 1);
    if(vResult.vt == 0)
    {
        vResult = currentObjExcelLink->callMethod(pwstProperty[0], pwstProperty[1], &vData, iArg);
    }
    else
    {
        vResult = currentObjExcelLink->callMethod(vResult, pwstProperty[nbInputArgument(pvApiCtx) - 2], &vData, iArg);
    }

    if(vResult.vt == VT_ERROR)
    {
        createScalarBoolean(pvApiCtx, nbInputArgument(pvApiCtx) + 1, 0);
    }
    else
    {
        createScalarBoolean(pvApiCtx, nbInputArgument(pvApiCtx) + 1, 1);
    }

    AssignOutputVariable(pvApiCtx, 1) = nbInputArgument(pvApiCtx) + 1;
    ReturnArguments(pvApiCtx);

    freeAllocatedMatrixOfWideString(1, iPropCount, pwstProperty);
    return 0;
}
/*--------------------------------------------------------------------------*/
